@@ -5,6 +5,7 @@ import uuid
 # local imports
 from .error import SessionExpiredError
 from .error import SessionError
+from .error import TokenExpiredError
 
 DEFAULT_AUTH_EXPIRE = 60
 DEFAULT_REFRESH_EXPIRE = 60 * 60 * 24 * 30
@@ -13,22 +14,19 @@ DEFAULT_REFRESH_EXPIRE = 60 * 60 * 24 * 30
 class Session:
     """State of authentication session and ACL data.
 
-    :param address: Ethereum address of user
-    :type address: str, 0x-hex
-    :param level: Authority level of user
-    :type level: int
-    :param items: ACL entries for user
-    :type items: dict
+    :param identity: Authorization digest of user
+    :type identity: str, 0x-hex
+    :param acl: ACL entries for user
+    :type acl: ecuth.acl.Acl
     """
-    def __init__(self, identity, level, items):
+    def __init__(self, identity, acl):
         self.identity = identity
-        self.level = level
-        self.items = items
         uu = uuid.uuid4()
         self.refresh = uu.bytes
         self.refresh_expire = time.time() + DEFAULT_REFRESH_EXPIRE
         self.auth = None
         self.auth_expire = 0.0
+        self.acl = acl
 
 
     def renew(self, refresh):
@@ -60,21 +58,35 @@ class Session:
         return self.auth_expire > time.time()
 
 
+    def read(self, item):
+        if not self.valid():
+            raise TokenExpiredError()
+        return self.acl.read(item)
+
+    def write(self, item):
+        if not self.valid():
+            raise TokenExpiredError()
+        return self.acl.write(item)
+
+    def val(self, item):
+        if not self.valid():
+            raise TokenExpiredError()
+        return self.acl.val(item)
+
+
+
     def __str__(self):
-        f = """level: {}
-identity: {}
+        f = """identity: {}
 token: {}
 expires: {}
-items:
+acl:
+{} 
 """
         d = f.format(
-            self.level,
             self.identity.hex(),
             self.auth.hex(),
             self.auth_expire,
+            self.acl,
         )
 
-        for k in self.items.keys():
-            d += "\t{}: {}\n".format(k, self.items[k])
-        
         return d
